use std::sync::Arc;

use axum::{extract::OriginalUri, routing::post, Extension, Json, Router};
use eyre::ContextCompat;
use foxchat::{
    fed,
    http::ApiError,
    model::{
        http::{channel::CreateMessageParams, guild::CreateGuildParams},
        Guild, Message,
    },
};
use serde::{de::DeserializeOwned, Serialize};
use tracing::debug;

use crate::{app_state::AppState, fed::ProxyServerHeader};

use super::auth::AuthUser;

pub fn router() -> Router {
    Router::new()
        .route("/guilds", post(proxy_post::<CreateGuildParams, Guild>))
        .route(
            "/channels/:id/messages",
            post(proxy_post::<CreateMessageParams, Message>),
        )
}

async fn proxy_get<R: Serialize + DeserializeOwned>(
    Extension(state): Extension<Arc<AppState>>,
    ProxyServerHeader(remote): ProxyServerHeader,
    AuthUser(user): AuthUser,
    OriginalUri(original_uri): OriginalUri,
) -> Result<Json<R>, ApiError> {
    let original = original_uri.to_string();
    let proxy_path = original
        .strip_prefix("/_fox/proxy/")
        .wrap_err("invalid url")?;
    println!("{}", proxy_path);

    let resp = fed::get::<R>(
        &state.private_key,
        &state.config.domain,
        &remote.domain,
        &format!("/_fox/chat/{}", proxy_path),
        Some(user.id),
    )
    .await?;

    Ok(Json(resp))
}

async fn proxy_post<B: Serialize, R: Serialize + DeserializeOwned>(
    Extension(state): Extension<Arc<AppState>>,
    ProxyServerHeader(remote): ProxyServerHeader,
    AuthUser(user): AuthUser,
    OriginalUri(original_uri): OriginalUri,
    Json(body): Json<B>,
) -> Result<Json<R>, ApiError> {
    let original = original_uri.to_string();
    let proxy_path = original
        .strip_prefix("/_fox/proxy/")
        .wrap_err("invalid url")?;
    debug!("Proxying request to {}", proxy_path);

    let resp = fed::post::<B, R>(
        &state.private_key,
        &state.config.domain,
        &remote.domain,
        &format!("/_fox/chat/{}", proxy_path),
        Some(user.id),
        &body,
    )
    .await?;

    Ok(Json(resp))
}
